import { Component, OnInit, SecurityContext, ViewChild } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { ActivatedRoute } from '@angular/router';
import { User } from '../../../models/user.model';
import { NotificationService } from '../../../services/notification.service';
import { OrientationService } from '../../../services/orientation.service';
import { TextareaComponent } from '../../../shared/components/textarea/textarea.component';
import { ProfileService } from '../../services/profile.service';

@Component({
  selector: 'app-orientation-details',
  templateUrl: './orientation-details.component.html',
  styleUrls: ['./orientation-details.component.scss'],
})
export class OrientationDetailsComponent implements OnInit {
  constructor(
    private route: ActivatedRoute,
    public orientationService: OrientationService,
    private notificationService: NotificationService,
    private profileService: ProfileService,
    private sanitizer: DomSanitizer,
  ) {}

  @ViewChild('comment', { static: false }) commentTextarea: TextareaComponent;

  public orientation;
  public isModalOpenned = false;
  public userProfile: User;
  public isEditMode = false;
  public from: string;
  public closingComment = '';
  public selectedStatus = '';

  ngOnInit(): void {
    this.route.queryParams.subscribe((params) => {
      this.from = params['from'];
    });

    this.route.params.subscribe((urlParams) => {
      const orientationId = urlParams.id || '';
      if (orientationId) {
        this.orientationService.getOrientation(orientationId).subscribe((orientation) => {
          this.orientation = orientation;
          this.acknowledgeNewOrientation();
          this.selectedStatus = this.orientation.status;
        });
      }
    });

    this.profileService.getProfile().then((profile) => {
      this.userProfile = new User(profile);
    });
  }

  public goBack(): void {
    history.back();
  }

  public showModal(): void {
    this.isModalOpenned = true;
  }

  public closeOrientation(event): void {
    if (event) {
      const comment = this.commentTextarea?.value || '';
      const sanitizedComment = this.sanitizer.sanitize(SecurityContext.HTML, comment);
      this.patchOrientation({
        status: this.selectedStatus,
        closingComment: sanitizedComment,
        closingDate: new Date(),
        modifiedBy: this.userProfile._id,
      });
    }
    this.isModalOpenned = false;
  }

  public acknowledgeNewOrientation(): void {
    if (this.orientation.status === 'new') {
      this.patchOrientation({
        status: 'acknowledged',
        modifiedBy: this.userProfile._id,
      });
    }
  }

  public editOrientation(): void {
    // TODO : edit orientation
    if (this.isEditMode) {
      // wip test patch
      this.patchOrientation({
        // status: 'completed',
        processingComment: 'processing comment',
        modifiedBy: this.userProfile._id,
      });
    }

    this.isEditMode = !this.isEditMode;
  }

  public setProcessingComment(comment): void {
    // TODO : patch comment
  }

  get isValidateDisabled(): boolean {
    return !this.selectedStatus || !this.commentTextarea?.value?.trim();
  }

  private patchOrientation(updatedFields): void {
    this.orientationService.patchOrientation(this.orientation._id, updatedFields).subscribe({
      next: (orientation) => {
        this.notificationService.showSuccess("L'orientation a bien été mise à jour.");
        // Display the updated values on the orientation details page
        this.orientation = orientation;
      },
      error: () => {
        this.notificationService.showErrorPleaseRetry(
          "Une erreur s'est produite lors de la mise à jour de l'orientation.",
        );
      },
    });
  }
}
